@@ -1,11 +1,11 @@
 #' Calculate the Reward for a POMDP Solution
-#' 
+#'
 #' This function calculates the expected total reward for a POMDP solution
 #' given a starting belief state.
-#' 
+#'
 #' The value is calculated using the value function stored in the POMDP
 #' solution.
-#' 
+#'
 #' @param x a POMDP solution (object of class POMDP).
 #' @param belief specification of the current belief state (see argument start
 #' in \code{\link{POMDP}} for details). By default the belief state defined in
@@ -18,33 +18,34 @@
 #' @author Michael Hahsler
 #' @seealso \code{\link{POMDP}}, \code{\link{solve_POMDP}}
 #' @examples
-#' 
+#'
 #' data("Tiger")
 #' sol <- solve_POMDP(model = Tiger)
-#' 
+#'
 #' # if no start is specified, a uniform belief is used.
 #' reward(sol)
-#' 
-#' # we have additional information that makes us believe that the tiger 
+#'
+#' # we have additional information that makes us believe that the tiger
 #' # is more likely to the left.
 #' reward(sol, belief = c(0.85, 0.15))
-#' 
+#'
 #' # we start with strong evidence that the tiger is to the left.
 #' reward(sol, belief = "tiger-left")
-#' 
-#' # Note that in this case, the total discounted expected reward is greater 
-#' # than 10 since the tiger problem resets and another game staring with 
+#'
+#' # Note that in this case, the total discounted expected reward is greater
+#' # than 10 since the tiger problem resets and another game staring with
 #' # a uniform belief is played which produces additional reward.
-#' 
+#'
 #' @export
 reward <- function(x, belief = NULL, epoch = 1) {
   .solved_POMDP(x)
   
-  if(is.null(belief)) belief <- x$model$start
+  if (is.null(belief))
+    belief <- x$model$start
   
   belief <- .translate_belief(belief, x)
- 
-  ## translate for converged POMDPs 
+  
+  ## translate for converged POMDPs
   e <- .get_pg_index(x, epoch)
   
   alpha <- x$solution$alpha[[e]]
@@ -54,7 +55,7 @@ reward <- function(x, belief = NULL, epoch = 1) {
   
   list(
     belief = belief,
-    reward = vs$reward, 
+    reward = vs$reward,
     pg_node = vs$pg_node,
     action = factor(pg$action[vs$pg_node], levels = x$model$actions)
   )
@@ -62,11 +63,16 @@ reward <- function(x, belief = NULL, epoch = 1) {
 
 ## this reward helper
 .rew <- function(belief, alpha) {
-  if(!is.matrix(belief)) belief <- rbind(belief)
-  r <- apply(belief, MARGIN = 1, FUN = function(b) {
-    rewards <- alpha %*% b
-    c(max(rewards), which.max(rewards))
-  })
+  if (!is.matrix(belief))
+    belief <- rbind(belief)
+  r <- apply(
+    belief,
+    MARGIN = 1,
+    FUN = function(b) {
+      rewards <- alpha %*% b
+      c(max(rewards), which.max(rewards))
+    }
+  )
   r <- as.data.frame(t(r))
   colnames(r) <- c("reward", "pg_node")
   #r$pg_node <- factor(r$pg_node, levels = 1:nrow(alpha))
