@@ -1,35 +1,35 @@
-## ----setup, include=FALSE------------------------------------------------
+## ----setup, include=FALSE-----------------------------------------------------
 knitr::opts_chunk$set(echo = TRUE)
 library("pomdp")
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 str(args(POMDP))
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  start = c(0.5 , 0.3 , 0.2)
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  start = "uniform"
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  start = 3
 #  start = c(1, 3)
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  start <- "state3"
 #  start <- c("state1" , "state3")
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  start = c("-" , "state2")
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  transition_prob = rbind(
 #    T_("action1", "state1", "state2", 0.1),
 #    T_("action2", "state1", "state3", 0.9),
 #    T_("*"      , "state2", "*",      1)
 #  )
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  transition_prob = list(
 #    "action1" = matrix(c(
 #        0.1, 0.4, 0.5,
@@ -47,7 +47,7 @@ str(args(POMDP))
 #        0.4, 0.4, 0.2), nrow = 3 , byrow = TRUE) ,
 #      "action2" = "uniform")
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  observation_prob = rbind(
 #    O_("*", "state1", "obs1", 0.1),
 #    O_("*", "state1", "obs2", 0.9),
@@ -57,7 +57,7 @@ str(args(POMDP))
 #    O_("*", "state3", "obs2", 0.6)
 #  )
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  observation_prob = list(
 #    "action1" = matrix(c(0.1, 0.9,
 #                         0.3, 0.7,
@@ -73,7 +73,7 @@ str(args(POMDP))
 #                        0.3, 0.7,
 #                        0.4, 0.6), nrow = 3, byrow = TRUE))
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  reward = rbind(
 #    R_("action1", "*", "state1", "*", 10000),
 #    R_("action1", "*", "state2", "*", 2000),
@@ -81,7 +81,7 @@ str(args(POMDP))
 #    R_("action2", "*", "state2", "*", 100)
 #  )
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  reward = list(
 #    "action1" = list(
 #       "state1" = matrix(c(1, 2, 3, 4, 5, 6) , nrow = 3 , byrow = TRUE),
@@ -92,13 +92,13 @@ str(args(POMDP))
 #       "state2" = matrix(c(0, 9, 8, 2, 5, 4) , nrow = 3 , byrow = TRUE),
 #       "state3" = matrix(c(4, 3, 4, 4, 5, 6) , nrow = 3 , byrow = TRUE)))
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 str(args(solve_POMDP))
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library("pomdp")
 
-TigerProblem <- POMDP(
+Tiger <- POMDP(
   name = "Tiger Problem",
   
   discount = 0.75,
@@ -128,25 +128,21 @@ TigerProblem <- POMDP(
   )
 )
 
-TigerProblem
+Tiger
 
-## ------------------------------------------------------------------------
-tiger_solved <- solve_POMDP(TigerProblem)
-tiger_solved
+## -----------------------------------------------------------------------------
+sol <- solve_POMDP(Tiger)
+sol
 
-## ------------------------------------------------------------------------
-solution(tiger_solved)
+## -----------------------------------------------------------------------------
+sol$solution
 
-## ----fig.width = 10, fig.asp = .7----------------------------------------
-plot(tiger_solved)
+## ----fig.width = 10, fig.asp = .7---------------------------------------------
+plot_policy_graph(sol)
 
-## ------------------------------------------------------------------------
-alpha <- solution(tiger_solved)$alpha
+## -----------------------------------------------------------------------------
+alpha <- sol$solution$alpha
 alpha
 
-plot(NA, xlim = c(0, 1), ylim = c(0, 20), xlab = "Belief space (for tiger is left)", 
-  ylab = "Value function")
-for(i in 1:nrow(alpha)) abline(a = alpha[i,2], b = alpha[i,1]- alpha[i,2], col = i, xpd = FALSE)
-legend("topright", legend = 
-    paste0(1:nrow(alpha),": ", solution(tiger_solved)$pg[,"action"]), col = 1:nrow(alpha), lwd=1)
+plot_value_function(sol, ylim = c(0,20))
 
